package htmlfixit;
#############################################################################################
# Output:
# 0 = "No action or auth params received."
# 1 = "Auth successful"
# 2 = "Auth attempt"
# 3 = "Auth Failed."
# 4 = "Action specified, no auth."
#
# Usage Example:
#my $query = new CGI;
#my ($login_status, $login_message, $session) = htmlfixit::login_code($query, 'auth_id', 'username', 
#                                                             'password', 'login',  
#                                                             $user_name, $pass_word, 
#                                                             $action_param, $tmp_dir,$domain, 
#                                                             $path, $expiration);
#
# What the parameters mean:
#- $query is the CGI object containing all passed form parameters.
#- auth_id is the name of the cookie that will be set on login and also can be passed as a form param.
#- username is the name of the form param on the login screen used to collect the username entered by the user.
#- password is the same as above accept in this case to collect the password.
#- login is the form param used to designate a login attempt.
#- $user_name is the actual username that is required for a login.
#- $pass_word is the actual password required for a successful login.
#- $action_param, a param used to designate an action to perform instead of a login attempt.
#  for example in the downloader script, setting $action to 'id' results in the script checking 
#  for an 'id' form param and if it receives one, it starts the download process instead of 
#  displaying a login.
#- $tmp_dir is a directory with write access by the script. traditionally /tmp on unix.
#- $domain, domain name required for setting of cookie.
#- $path, part to the script, used for the cookie, eg 'cgi-bin/somedir/'
#- $expiration, to set the cookie expiration period, default is session cookie.

#if    ($login_status == 0){htmlfixit::not_specified_error(); goto _end;}
#elsif ($login_status == 1){&display_downloads($query,$counterfiledir,'dl_admin',$domain,$path,$session,$tmp_dir, $demo); goto _end;}
#elsif ($login_status == 2){htmlfixit::display_login('Downloader', 'dl.pl', 'login=yes', $login_message,$demo,$user_name,$pass_word); goto _end;}
#elsif ($login_status == 3){htmlfixit::display_login('Downloader', 'dl.pl', 'login=yes', $login_message,$demo,$user_name,$pass_word); goto _end;}
#elsif ($login_status == 4){what_to_do($query, $counterfiledir, $download_url); goto _end;}
#else {goto _end;}
#_end:
################################

sub login_code
{
my ($query, $cookie_id, $username_param, $password_param, $auth_param, $set_user,
    $set_passwd, $action_param, $temp_dir, $domain, $path, $expiration) = @_;

my $status                =  0;
my $logging_in            =  0;
my $logged_in             =  0;
my $session_file          = '';
my $dirty_session_file    = '';

my $dirty_message         = ($query->param('message')) || '';
my $message = $1 if ($dirty_message =~ m/^([\w \.]*)$/i);

my $session_file_and_path = '';
my $auth_string           = '';

  # Don't waste time doing all this if it isn't a login attempt.
if ($query->param("$auth_param")){

########## Cleanup old unused sessions. ############
opendir(CLEANUP,"$temp_dir")|| die "Can't open $temp_dir Reason: $!";
my @cleanup_files = sort(grep(/fixit_ses_\.*/,readdir(CLEANUP)));
closedir(CLEANUP) || die "Can't close $temp_dir Reason: $!";
foreach my $old_files (@cleanup_files)
        {
        my $sess_file = 'not_a_session_file';
           if ($old_files =~ m/^(fixit_ses_[\.\w]*\.fud)$/i)
              {$sess_file = $1;}        
        
         if ($sess_file ne 'not_a_session_file')
         {
         my $temp_path = $temp_dir.'/'.$sess_file;
           if (-M $temp_path >= 1)
           {
           unlink $temp_path or die "can't delete $temp_path reason $!";
           }
         }
        }
########## End of cleanup ##########################
my %cookie = fetch CGI::Cookie; 

      my $auth_string = crypt($set_passwd, (substr($set_user, 0, 2)));

      if ($query->param("$cookie_id"))
         {
         $dirty_session_file = $query->param("$cookie_id");
                  if ($dirty_session_file =~ m/^(fixit_ses_[\.\w]*)$/i)
                  {
                  $session_file = $1; 
                  $session_file_and_path = ($temp_dir.'/'.$session_file);
                  }
                  else
                  {
                  #just assign the file to a non existant file that matches the regex.
                  $session_file='fixit_ses';
                  $session_file_and_path = ($temp_dir.'/'.$session_file);
                  }
         }
   elsif  ($cookie{"$cookie_id"})
          {
          $dirty_session_file = $cookie{"$cookie_id"}->value || 'fixit_ses';
          
                  if ($dirty_session_file =~ m/(^fixit_ses_[\.\w]*)$/i)
                  {
                  $session_file = $1; 
                  }
                  else
                  {
                  $session_file='fixit_ses';
                  } 
          $session_file_and_path = ($temp_dir.'/'.$session_file);
          }

      else{
          $session_file=&create_sessionfile($set_passwd, $temp_dir);
          &set_cookie($cookie_id,$session_file,$expiration,$domain,$path);
          $session_file_and_path = ($temp_dir.'/'.$session_file);
          }
    unless (-e $session_file_and_path)
           {
           $session_file =&create_sessionfile($set_passwd, $temp_dir);
           &set_cookie($cookie_id,$session_file,$expiration,$domain,$path);
           unless($message ne '')
           {
           $message = 'Session has expired, please login again.';
           }
           $status = 2;
           return ($status, $message, $session_file);           
           }          
                      #  Expire the session if its over 24 hours old, then create a new one.
             if (-M $session_file_and_path >= 1)
                {
                unlink $session_file_and_path or die "can't delete $session_file_and_path reason $!";
                # Old session expired, create new one.
                $session_file =&create_sessionfile($set_passwd, $temp_dir);
                &set_cookie($cookie_id,$session_file,$expiration,$domain,$path);
                $message = 'Session Expired, please login again.';
                $status = 2;
                return ($status, $message, $session_file);
                }     
          
########################## CHECK SESSION FILE AUTH HERE #################           
        open (SESSION, "$session_file_and_path") || die "can't open file: $session_file_and_path reason: $!";
        flock(SESSION, 1) or die "cannot lock $session_file_and_path $!";
        my $hash = <SESSION>;
        flock(SESSION, 8);
        close (SESSION);
           $hash = 'nope|' unless ($hash);
           my @session_data = split(/\|/, $hash);
           # the retrieved hash should be the first item in the array.
                if ($auth_string eq $session_data[0])
                {
                   if (($query->param('logout')))
                   {
                   ($status, $message) = &recreate_session($temp_dir,$session_file,$cookie_id,$expiration,$domain,$path,$set_passwd);
                    $status = 2;
                    return ($status, $message, $session_file);           
                   }                 
                $status = 1;
                return ($status, $message, $session_file); 
                }
}  # end of if ($query->param($auth_param))

        if ($query->param("$auth_param"))
           {
             unless($query->param('displayed')) 
                   {
                   $message = 'Please enter username and password';
                   $status = 2;
                   return ($status, $message, 'undef');
                   }
           
                if (($query->param("$username_param") eq '') || ($query->param("$password_param") eq ''))
                   {
                   $message = 'Missing or incorrect username or password';
                   $status = 2;
                   return ($status, $message, 'undef');
                   }
           }
 
     if (($query->param("$auth_param"))  && ($query->param("$username_param")) && ($query->param("$password_param")))       
           {
            if (($query->param("$username_param") eq $set_user) && ($query->param("$password_param") eq $set_passwd))
               {
               $status = 1;
               # User has logged in successfully, and now we set the login param to the auth file.
               # the authstring is based on a random number multiplied by unix time and using part 
               # of the username as the salt.
               my $authstring = crypt($set_passwd, (substr($set_user, 0, 2)));
               &add_authdata($authstring, $session_file, $temp_dir);
               return ($status, $message, $session_file);
               }
          else {
               $status = 3;
               $message = 'Missing or incorrect username or password';
               return ($status, $message);
               }
           }
           
        if ($query->param("$action_param"))
           {
           $status = 4;
           }
# If none of the above are true, then $status is still 0, so all is well.        
return ($status, $message);
} # End of sub.

################################################################################################
###################################### AUTH SUBS ###############################################
# usage create_sessionfile($set_passwd, $temp_dir); 
sub create_sessionfile
{
my ($passwd,$temp_dir) = @_;
my $clean_sessh_name = '';

# make a 20 char long string of random chars.
my @characters=('a'..'z');
my $rand_string;

for(1..20) 
{
$rand_string.= $characters[(rand(scalar(@characters)))];
}
my $salty = substr($passwd, 1, 3); # Use the first two chars of the passwd as the salt for crypt.
my $session_name = 'fixit_ses_'.(rand(999999999)*time).crypt($rand_string , $salty);
   $session_name =~ tr/\+//d;
   $session_name =~ tr/\.//d;
   $session_name = lc($session_name).'.fud';
   
   $clean_sessh_name = $1 if ($session_name =~ m/^(fixit_ses_[\.\w]*)$/i);

if ($clean_sessh_name ne '')
        {
        my $sessh = $temp_dir.'/'.$clean_sessh_name;
        open (SESSION, "+>>$sessh") || die "can't open file: $sessh reason: $!";
        flock(SESSION, 2) or die "cannot lock $sessh : $!";
        flock(SESSION, 8);
        close (SESSION);
        }
return $clean_sessh_name;       
}
################################################################################################
sub add_authdata
{
my ($auth, $sess_file, $temp_dir) = @_;
my $sessh = $temp_dir.'/'.$sess_file;
open (SESSION, ">$sessh") || die "can't open file: $sessh reason: $!";
   flock(SESSION, 2) or die "cannot lock $sessh : $!";
   print SESSION $auth.'|'."\n";
   flock(SESSION, 8);
   close (SESSION);
return 1;
}
################################################################################################
#Usage: &remove_session($temp_dir,$session_file,$cookie_id,$expiry_date,$domain,$path)
sub recreate_session
{
my ($temp_dir,$session_file,$cookie_id,$expiration,$domain,$path,$set_passwd) = @_;
     my $logout_session = $temp_dir.'/'.$session_file;
     if (-f $logout_session)
     {unlink $logout_session or die "can't delete $logout_session reason $!";}
     $message = 'Logout successful';
     $session_file=&create_sessionfile($set_passwd, $temp_dir);
     &set_cookie($cookie_id,$session_file,$expiration,$domain,$path);
     $status = 2;
     return ($status, $message);
}
################################################################################################
################################################################################################
#usage my $css = display_css();

sub display_css
{
my $csstags = qq(
<style type='text/css'>
<!--
body    {
        font:12px/1.2 Arial, sans-serif, Helvetica;
        background:#ddd;
        text-align: center;
        padding:0px;
        margin:0px;
        }

a       {
        text-decoration:none;
        font-weight:none;
        color:#c00;
        }
a:hover {
        color: black;
        }
        
.heading{
        font:20px/1.2 Arial, sans-serif, Helvetica;
        font-weight: bold;
        }
.smallheading{
        font:14px/1.2 Arial, sans-serif, Helvetica;
        font-weight: bold;
        }


.content {
         font:13px/1.2 Arial, sans-serif, Helvetica;
         position:relative;
         margin-left: auto; 
         margin-right: auto;
         background:#fff;
         border:1px solid #000;
         width: 620px;
         }
         
table.cont {
            border:1px solid black;
           }

table.cont td {
               border-bottom: 0px solid black;
               border-top:    1px solid black;
               border-left:   0px none;
               border-right:  0px none; 
              }
table.cont tr.evn {
                  background-color: #FFFFFF;
                  }
table.cont tr.odd { 
                  background-color: #99CCCC;
                  } 
//-->
</style>         
);

return $csstags;
}

################################################################################################
# Usage:
# htmlfixit::display_login('App_name', 'script_name.pl', 'login_params', $login_message,$demo,$user_name,$pass_word);
# Where:
#-App_name is the name of the program to be displayed on the login screen.
#-script_name.pl is the name of the script executable itself.
#-login_params is the form parameters used to designate a login attempt.
#- $login_message any message passed though this will displayed one the login screen.
#- $demo, used to create a demo mode where the value of $demo determines if the login screen is prefilled out or not.
#  possible values 0 or 1. (1 being demo mode.)
#-$user_name is the scripts real login username, used to fill in form fields in demo mode.
#-$pass_word is the scripts real login password, again used to fill in login form field.
# care MUST be taken to ensure demo mode is off for normal usage else your user/password will be displayed for all to see.

sub display_login
{
my $subject     = shift;
my $script_name = shift;
my $params      = shift;
my $message     = shift;
my $demo        = shift;
my $user_name   = shift; 
my $pass_word   = shift; 
my $css = &display_css();


my $message_display = '<br /><span align=\'center\' class=\'heading\'>Please login to access the admin console</span><br />';
unless ($message eq '')
{$message_display = '<br /><span align=\'center\' class=\'heading\'>'.$message.'</span><br />';}

my ($login_user, $login_pass);
if ($demo == 1)
{
$login_user = '<input type=\'text\' name=\'username\' value=\''.$user_name.'\' />';
$login_pass = '<input type=\'password\' name=\'password\' value=\''.$pass_word.'\' />';
}

elsif($demo == 0)
{
$login_user = '<input type=\'text\' name=\'username\' />';
$login_pass = '<input type=\'password\' name=\'password\' />';
}
else
{
$login_user = '<input type=\'text\' name=\'username\' />';
$login_pass = '<input type=\'password\' name=\'password\' />';
}

my $login = qq{
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html><head>
<title>HTMLfixIT.com Statistical page counter for HTMLfixIT.com.</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />
<meta http-equiv='Cache-Control' content='no-store' />
<meta http-equiv='pragma' content='no cache' />
<meta http-equiv='Expires' content='0' />
$css
</head>
<body>
<br />
<div align='center' class='content' id='login'> 
<br /><span align='center' class='heading'><a href='http://HTMLfixIT.com' target='_blank'>HTMLfixIT.com</a> $subject Admin Login.</span>
<br /><br />
</div>
<br />
<div align='center' class='content'>
$message_display
<br />
<form action='$script_name?$params' method='post'>
<!-- <form action='$script_name' method='post'> //-->
  <table align='center' border='0'>
   <tr>
      <td>UserName: </td>
      <td>$login_user</td>
   </tr>
   <tr>
      <td>PassWord: </td>
      <td>$login_pass
          <input type='hidden' name='login' value='yes' />
          <input type='hidden' name='displayed' value='yes' />
      </td>
   </tr>
   <tr>
      <td colspan='2' align='center'><input type='submit' name='submit' value='Click to enter' /></td>
   </tr>
</table>
</form>
</div>
</body>
</html>
              };
print "Content-type: text/html\n\n";              
print $login;
}
################################################################################################
#usage: htmlfixit::login_error('script_name');
# script_name is simply the name of the script, used so that the 'try again' link will function.
#
# This sub is used simply to display a failed login message where needed.
# generally this is now depreciated in favour of passing a message to the display_login sub. 
sub login_error
{
   my $script_name = shift;
   my $css = &display_css();

   my $login_error = qq{
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html><head><title>Error!</title></head>
$css
<body><br /><br />
<div align='center' class='content'>   
 <h3>Sorry, your username and/or password is incorrect.</h3>
 <h3>please try again:</h3>
 <p><a href='$script_name?login=yes'>Try again.</a></p>
  </div>
 </body>
</html>
               };
             
print "Content-type: text/html\n\n";              
print $login_error;        
        
}
###############################################################################################
#usage htmlfixit::not_specified_error('script name');
# I believe good scripts should always specify explicitly what action is required rather then to
# make assumptions on the users behalf. This small sub simply displays an error screen when no 
# auth or action params have been passwed to the script.

sub not_specified_error
{
   my $script_name = shift;
   my $css = &display_css();
   
  my $error = qq{
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html><head><title>Error!, No action specified.</title></head>
$css
 <body><br /><br />
   <div align='center' class='content'> 
   <h3>Sorry, you have not specified any action to perform.</h3>
   <h3>I don't know what you want me to do.</h3>
   </div>
 </body>
</html>
               };

print "Content-type: text/html\n\n";              
print $error;    
}

###############################################################################################
# usage: htmlfixit::voting('product name', 'voting page address');
# Used by htmlfixit scripts to display a voting banner in the admin console of the scripts.
sub voting
{
my $product = shift;
my $url     = shift;
   $output =  "\n".'<br /><div align=\'center\' class=\'content\' id=\'vote\'><br /><span align=\'center\' class=\'smallheading\'><b>PLEASE VOTE FOR US.</b></span><br />If you like this application, please help us let others know it exists by ';
   $output .= "\n".'<br />voting for it. To Vote, simply go to this page: <a href=\''. $url.'\' target=\'_blank\'>Vote for '.$product.'</a><br /><br /></div>'."\n";
return $output;
}

###############################################################################################
# Usage: my $date = htmlfixit::get_date();
#
# returns a nice pretty formatted user friendly date display.
sub get_date
{
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst,$date);
        my (@days, @months);  
            @days = ('Sunday','Monday','Tuesday','Wednesday','Thursday', 'Friday','Saturday');
            @months = ('January','February','March','April','May','June','July',
                       'August','September','October','November','December');
           ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);        
           if ($hour < 10)         
           { 
           $hour = "0$hour";
           }
           if ($min < 10) 
           { 
           $min = "0$min"; 
           }
           if ($sec < 10) 
           {
           $sec = "0$sec"; 
           }
           
           # Change 24 hour time to 12.
           if ($hour > 12)
              {
              my $dif = ($hour - 12);
              $hour = ($dif);
              $hour = 12 if $hour == 0;
              $hour = ($hour . ':' . $min . ' PM');
              }
           else {$hour = ($hour . ':' . $min . ' AM');}               

           $year += 1900;
           $date = "$days[$wday], $months[$mon] $mday, $year at $hour";
           return $date;             
} 
###############################################################################################
sub set_cookie 
{
# usage: #&set_cookie($name, $value, $expiration,$domain,$path);
#
# This sub uses the CGI modules cookie handling methods to set a 
# cookie to the users browser when they log into the admin console.
# Its important to NOT send any other headers until you have set the 
# cookie, the cookie header MUST be sent before any html header.
my ($name, $value, $expiration, $domain, $path) = @_;
my $secure = '';

                my $cookie = new CGI::Cookie(
                                             -name    => $name,
                                             -value   => $value, 
                                             -path    => $path,
                                             -domain  => $domain, 
                                             -expires => $expiration 
                                             );                              

        print "Set-Cookie: $cookie\n";
        #print "Content-type: text/html\n";

               }                
###############################################################################################
1;
__END__